@@ -39,6 +39,9 @@
 
 HINSTANCE g_libCEC = NULL;
 
+/*!
+ * @deprecated Please use LibCecInit() instead
+ */
 CEC::ICECAdapter *LoadLibCec(const char *strName, CEC::cec_logical_address iLogicalAddress = CEC::CECDEVICE_PLAYBACKDEVICE1, uint16_t iPhysicalAddress = CEC_DEFAULT_PHYSICAL_ADDRESS, const char *strLib = NULL)
 {
   if (!g_libCEC)
@@ -54,6 +57,32 @@ CEC::ICECAdapter *LoadLibCec(const char *strName, CEC::cec_logical_address iLogi
   return static_cast< CEC::ICECAdapter* > (CreateLibCec(strName, (uint8_t) iLogicalAddress, iPhysicalAddress));
 }
 
+/*!
+ * @brief Create a new libCEC instance.
+ * @param strDeviceName The name of the primary device to pass to other CEC devices.
+ * @param types The list of device types to register on the bus.
+ * @param strLib The name of and/or path to libCEC
+ * @return An instance of libCEC or NULL when it failed to load.
+ */
+CEC::ICECAdapter *LibCecInit(const char *strDeviceName, CEC::cec_device_type_list types, const char *strLib = NULL)
+{
+  if (!g_libCEC)
+    g_libCEC = LoadLibrary(strLib ? strLib : "libcec.dll");
+  if (!g_libCEC)
+    return NULL;
+
+  typedef void* (__cdecl*_LibCecInit)(const char *, CEC::cec_device_type_list);
+  _LibCecInit LibCecInit;
+  LibCecInit = (_LibCecInit) (GetProcAddress(g_libCEC, "CECInit"));
+  if (!LibCecInit)
+    return NULL;
+  return static_cast< CEC::ICECAdapter* > (LibCecInit(strDeviceName, types));
+}
+
+/*!
+ * @brief Destroy an instance of libCEC.
+ * @param device The instance to destroy.
+ */
 void UnloadLibCec(CEC::ICECAdapter *device)
 {
   typedef void (__cdecl*_DestroyLibCec)(void * device);
@@ -72,6 +101,9 @@ void UnloadLibCec(CEC::ICECAdapter *device)
 
 void *g_libCEC = NULL;
 
+/*!
+ * @deprecated Please use LibCecInit() instead
+ */
 CEC::ICECAdapter *LoadLibCec(const char *strName, CEC::cec_logical_address iLogicalAddress = CEC::CECDEVICE_PLAYBACKDEVICE1, uint16_t iPhysicalAddress = CEC_DEFAULT_PHYSICAL_ADDRESS, const char *strLib = NULL)
 {
   if (!g_libCEC)
@@ -96,13 +128,55 @@ CEC::ICECAdapter *LoadLibCec(const char *strName, CEC::cec_logical_address iLogi
   _CreateLibCec* CreateLibCec = (_CreateLibCec*) dlsym(g_libCEC, "CECCreate");
   if (!CreateLibCec)
   {
-    cout << "cannot find CreateLibCec" << endl;
+    cout << "cannot find CECCreate" << endl;
     return NULL;
   }
 
   return (CEC::ICECAdapter*) CreateLibCec(strName, iLogicalAddress, iPhysicalAddress);
 }
 
+/*!
+ * @brief Create a new libCEC instance.
+ * @param strDeviceName The name of the primary device to pass to other CEC devices.
+ * @param types The list of device types to register on the bus.
+ * @param strLib The name of and/or path to libCEC
+ * @return An instance of libCEC or NULL when it failed to load.
+ */
+CEC::ICECAdapter *LibCecInit(const char *strDeviceName, CEC::cec_device_type_list types, const char *strLib = NULL)
+{
+  if (!g_libCEC)
+  {
+#if defined(__APPLE__)
+    g_libCEC = dlopen(strLib ? strLib : "libcec.dylib", RTLD_LAZY);
+#else
+    g_libCEC = dlopen(strLib ? strLib : "libcec.so", RTLD_LAZY);
+#endif
+    if (!g_libCEC)
+    {
+#if defined(__APPLE__)
+      cout << "cannot find " << (strLib ? strLib : "libcec.dylib") << dlerror() << endl;
+#else
+      cout << "cannot find " << (strLib ? strLib : "libcec.so") << dlerror() << endl;
+#endif
+      return NULL;
+    }
+  }
+
+  typedef void* _LibCecInit(const char *, CEC::cec_device_type_list);
+  _LibCecInit* LibCecInit = (_LibCecInit*) dlsym(g_libCEC, "CECInit");
+  if (!LibCecInit)
+  {
+    cout << "cannot find CECInit" << endl;
+    return NULL;
+  }
+
+  return (CEC::ICECAdapter*) LibCecInit(strDeviceName, types);
+}
+
+/*!
+ * @brief Destroy an instance of libCEC.
+ * @param device The instance to destroy.
+ */
 void UnloadLibCec(CEC::ICECAdapter *device)
 {
   typedef void* _DestroyLibCec(CEC::ICECAdapter *);
